'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.clearHtml = exports.getHtml = exports.splitListWith = exports.filterListWith = exports.mapListWith = exports.deduplicate = exports.flatten = exports.getList = exports.hasClassEq = exports.getClass = exports.hasAttrEq = exports.hasAttr = exports.protectArray = exports.everyToArray = exports.getAttr = exports.removeEmpty = exports.filterWith = exports.hasTag = exports._getFirst = exports.nothing = exports.emptyList = undefined;

var _pytils = require('pytils');

var _main = require('./main');

var emptyList = exports.emptyList = function emptyList() {
  return [];
};

var nothing = exports.nothing = function nothing() {
  return undefined;
};

var _getFirst = exports._getFirst = function _getFirst(list) {
  return (0, _pytils.isArray)(list) ? list[0] : undefined;
};

var hasTag = exports.hasTag = function hasTag(name) {
  return function (node) {
    return (0, _pytils.path)(['tag'], node) === name;
  };
};

var filterWith = exports.filterWith = function filterWith(list, With) {
  return (0, _pytils.path)(['filter'], list) ? list.filter(With) : list;
};

var removeEmpty = exports.removeEmpty = function removeEmpty(list) {
  return (0, _pytils.isArray)(list) ? filterWith(list, Boolean) : list;
};

var getAttr = exports.getAttr = function getAttr(key) {
  switch (key) {
    case 'text':
      return function (node) {
        return (0, _pytils.path)(['text'], node);
      };

    case 'class':
      return function (node) {
        return (0, _pytils.path)(['class'], node);
      };

    default:
      return key ? function (node) {
        return (0, _pytils.path)(['attrs', key], node);
      } : nothing;
  }
};

var everyToArray = exports.everyToArray = function everyToArray(list) {
  return (0, _pytils.isArray)(list) ? list : [list];
};

var protectArray = exports.protectArray = function protectArray(fx) {
  return function (list) {
    return (0, _pytils.isArray)(list) ? list.map(function (list) {
      return fx(list);
    }) : fx(list);
  };
};

var hasAttr = exports.hasAttr = function hasAttr(key) {
  var _getAttr = getAttr(key);
  return function (node) {
    return !!_getAttr(node);
  };
};

var hasAttrEq = exports.hasAttrEq = function hasAttrEq(key, value) {
  var _getAttr = getAttr(key);
  return function (node) {
    return _getAttr(node) === value;
  };
};

var getClass = exports.getClass = getAttr('class');

var hasClassEq = exports.hasClassEq = function hasClassEq(value) {
  return function (node) {
    var Class = getClass(node);
    return (0, _pytils.path)(['indexOf'], Class) ? Class.indexOf(value) >= 0 : false;
  };
};

var getList = exports.getList = function getList(direction) {
  return function (node) {
    return node.link && node.link[direction] ? node.link[direction] : [];
  };
};

var flatten = exports.flatten = function flatten(Lists) {
  if ((0, _pytils.isArray)(Lists)) {
    var flat = [];
    Lists.map(function (list) {
      return flat = flat.concat(list);
    });
    return flat;
  } else {
    return Lists;
  }
};

var deduplicate = exports.deduplicate = function deduplicate(duplicate) {
  if (duplicate && duplicate.length && duplicate[0].uniq >= 0) {
    var uniqs = [];
    var list = [];
    duplicate.map(function (node) {
      if (uniqs.indexOf(node.uniq) < 0) {
        uniqs.push(node.uniq);
        list.push(node);
      }
    });
    return list;
  } else {
    return duplicate;
  }
};

var mapListWith = exports.mapListWith = function mapListWith(process) {
  return function (list) {
    return everyToArray(list).map(process);
  };
};

var filterListWith = exports.filterListWith = function filterListWith(process) {
  return function (list) {
    return everyToArray(list).filter(process);
  };
};

var splitListWith = exports.splitListWith = function splitListWith(process) {
  return function (list) {
    return everyToArray(list).reduce(function (acc, node) {
      process(node) ? acc.with.push(node) : acc.not.push(node);
      return acc;
    }, { with: [], not: [] });
  };
};

var getHtml = exports.getHtml = protectArray(function (node) {
  var start = (0, _pytils.path)(['start'], node);
  var end = (0, _pytils.path)(['end'], node);
  if (start >= 0 && end >= start) {
    var file = (0, _pytils.path)(['obj', 'file'], _main.html);
    if ((0, _pytils.isString)(file)) {
      return file.substr(start, end - start);
    }
  }
  return node;
});

var removeScripts = /<script[\w\W]*?script>/gim;

var clearHtml = exports.clearHtml = protectArray(function (html) {
  return (0, _pytils.path)(['replace'], html) ? html.replace(removeScripts, '') : html;
});