'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.readQuery = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _pytils = require('pytils');

var _compositions = require('./compositions');

var _main = require('./main');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var spaces = '\\s+';
var anythings = '[\\w\\W]+';
var withKeys = 'all-childs|first-childs|all-below|first-below|all|first';
var onlyKey = spaces + '(?:\\\'(' + anythings + '?)\\\')';
var keyAndValue = spaces + '(?:\\\'(' + anythings + '?)\\\')(?:' + spaces + '==' + spaces + '\\\'(' + anythings + ')\\\')?';
var instructionParse = [{ type: 'get(onlyType)', regex: new RegExp('^get' + spaces + '(first|text|class|dangerouns-html|html)$', 'i') }, { type: 'get(typeAndName)', regex: new RegExp('^get' + spaces + '(list|attr)(?:' + spaces + '\\\'(' + anythings + ')\\\')?$', 'i') }, { type: 'with(onlyName)', regex: new RegExp('^(' + withKeys + ')' + spaces + 'with' + spaces + '(text|tag|class)(?:' + onlyKey + ')?$', 'i') }, { type: 'with(nameAndValue)', regex: new RegExp('^(' + withKeys + ')' + spaces + 'with' + spaces + '(attr)(?:' + keyAndValue + ')?$', 'i') }];

var instructionErro = function instructionErro(instruction) {
  throw '"' + instruction + '" not is a valid instruction';
};

var queryErro = function queryErro(query) {
  throw '"' + query + '" not is a valid query';
};

var clearAndSplit = function clearAndSplit(query) {
  if (!query || typeof query !== 'string') {
    queryErro(query);
  }
  query = query.replace(/\n\s+\n?/gm, '\n').split('\n').filter(Boolean);
  if (query.length <= 0) {
    queryErro(query);
  }
  return query;
};

var getResults = function getResults(inst) {
  return (0, _pytils.toArray)(inst.result).map(function (n) {
    return (0, _pytils.path)(['toLowerCase'], n) && n.toLowerCase();
  });
};

var createGet = function createGet(inst) {
  var typeAndName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'onlyType';

  var _getResults = getResults(inst),
      _getResults2 = _slicedToArray(_getResults, 2),
      type = _getResults2[0],
      name = _getResults2[1];

  if (typeAndName === 'onlyType') {
    return createGetOnlyType(inst, type);
  } else {
    !name && instructionErro(inst.instruction);
    return createGetTypeAndName(inst.instruction, type, name);
  }
};

var createGetOnlyType = function createGetOnlyType(instruction, type) {
  switch (type) {
    case 'first':
    case 'text':
    case 'class':
    case 'html':
    case 'dangerouns-html':
      return (0, _compositions.get)(type);

    default:
      instructionErro(instruction);
  }
};

var createGetTypeAndName = function createGetTypeAndName(instruction, type, name) {
  switch (type) {
    case 'list':
    case 'attr':
      return (0, _compositions.get)(type, name);

    default:
      instructionErro(instruction);
  }
};

var createWith = function createWith(inst) {
  var nameAndValue = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'onlyName';

  var _getResults3 = getResults(inst),
      _getResults4 = _slicedToArray(_getResults3, 4),
      action = _getResults4[0],
      type = _getResults4[1],
      name = _getResults4[2],
      value = _getResults4[3];

  if (!type) {
    instructionErro(inst.instruction);
  }
  if (nameAndValue !== 'onlyName' && !name) {
    instructionErro(inst.instruction);
  }
  switch (action) {
    case 'all':
      return (0, _compositions.all)(type, name, value);

    case 'first':
      return (0, _compositions.first)(type, name, value);

    case 'all-childs':
      return (0, _compositions.allChilds)(type, name, value);

    case 'first-childs':
      return (0, _compositions.firstChilds)(type, name, value);

    case 'all-below':
      return (0, _compositions.allBelow)(type, name, value);

    case 'first-below':
      return (0, _compositions.firstBelow)(type, name, value);

    default:
      instructionErro(inst.instruction);
  }
};

var readInstruction = function readInstruction(instructionStr) {
  var instruction = instructionParse.map(function (rule) {
    return {
      type: rule.type,
      result: rule.regex.exec(instructionStr),
      instruction: instructionStr
    };
  }).filter(function (n) {
    return n.result;
  }).pop();
  switch (instruction && instruction.type) {
    case 'get(onlyType)':
      return createGet(instruction);

    case 'get(typeAndName)':
      return createGet(instruction, 'typeAndName');

    case 'with(onlyName)':
      return createWith(instruction);

    case 'with(nameAndValue)':
      return createWith(instruction, 'nameAndValue');

    default:
      instructionErro(instructionStr);
  }
};

var queryEngine = function queryEngine(compiled) {
  var toAsk = _pytils.composeDown.apply(undefined, _toConsumableArray(compiled));
  return function (htmlObj) {
    (0, _main.throwIfHtmlInvalid)(htmlObj);
    return toAsk(htmlObj.list);
  };
};

var readQuery = exports.readQuery = function readQuery(query) {
  try {
    var compiled = clearAndSplit(query).map(readInstruction);
    return queryEngine(compiled);
  } catch (erro) {
    throw 'fx-query: ' + erro;
  }
};